//++
// File Name:
//		hardware.h
//
// Contents:
//		Hardware-specific details
//--
#ifndef __XX_HARDWARE__
#define __XX_HARDWARE__

//
// Default values for hardware
//
#define XX_DEF_PORT		0x03BC
#define XX_DEF_PORT_SPAN	3
#define XX_DEF_IRQ			2
#define XX_DEF_DMA			3

//
// Define device registers as relative offsets
//
#define PAR_DATA		0 // Data register
#define PAR_STATUS		1 // Status register
#define PAR_CONTROL		2 // Control register

//
// Status register bits:
//
#define PAR_IRQ 	0x04	// 0 Interrupt request
#define PAR_ERR		0X08	// 0 Error
#define PAR_SELCTD	0x10	// 1 Printer selected
#define PAR_NOPAPR	0x20	// 1 Printer out of paper
#define PAR_ACK		0x40	// 0 Acknowledge
#define PAR_BSY		0x80	// 0 Printer busy

//
// Control register bits:
//
#define PAR_STROBE	0x01	// 1 Strobe
#define PAR_AUTOLF	0x02	// 1 Automatic line feed
#define PAR_INIT	0x04	// 0 Initialize printer
#define PAR_SELECT	0x08	// 1 Select printer
#define PAR_INTENB	0x10	// 1 Enable interrupts

//
// Define access macros for registers. Each macro
// takes a pointer to a Device Extension as an 
// argument
//
#define ParWriteData( pDevExt, bData )		\
(WRITE_PORT_UCHAR(							\
  pDevExt->PortBase + PAR_DATA, bData ))

#define ParReadStatus( pDevExt )			\
(READ_PORT_UCHAR(							\
  pDevExt->PortBase + PAR_STATUS ))

#define ParWriteControl( pDevExt, bData )	\
(WRITE_PORT_UCHAR(						\
  pDevExt->PortBase + PAR_CONTROL, bData ))

#endif // __XX_HARDWARE__

