//++
// File Name:
//		timer.c
//
// Contents:
//		Device time-out code
//--

//
// Driver-specific header files...
//
#include "xxdriver.h"

//
// Forward declarations of local functions
//
static BOOLEAN
XxProcessTimerEvent(
	IN PDEVICE_EXTENSION pDE
	);


//++
// Function:
//		XxIoTimer
//
// Description:
//		The system calls this function once every
//		second. If an I/O is in progress, this 
//		function decrements a time-out counter and
//		fails the request when the counter hits zero.
//
// Arguments:
//		Pointer to a Device object
//		Context (pointer to Device Extension)
//
// Return Value:
//		(none)
//--
VOID
XxIoTimer(
	IN PDEVICE_OBJECT DeviceObject,
	IN PVOID Context
	)
{
	PDEVICE_EXTENSION pDE = Context;

	if( pDE->TimeRemaining == -1 ) return;

	else if( !KeSynchronizeExecution(
				pDE->pInterrupt,
				XxProcessTimerEvent,
				pDE ))
	{
		//
		// Call the DPC routine to figure out a
		// final status and complete the IRP.
		//
		XxDpcForIsr(
			NULL,
			DeviceObject,
			DeviceObject->CurrentIrp,
			pDE );
	}
}

//++
// Function:
//		XxProcessTimerEvents
//
// Description:
//
// Arguments:
//		Pointer to a Device Extension)
//
// Return Value:
//		TRUE  -- TimeRemaining is >0 or == -1
//		FALSE -- TimeRemaining hit 0
//
//--
static BOOLEAN
XxProcessTimerEvent(
	IN PDEVICE_EXTENSION pDE
	)
{
	//
	// Test the timer again. It may have changed
	// while we were waiting for the Interrupt
	// spin lock. If so, do nothing.
	//
	if( pDE->TimeRemaining == -1 ) return TRUE;

	//
	// Decrement and test the timer.
	//
	if( --pDE->TimeRemaining > 0 ) return TRUE;

	//
	// A time-out has ocurred. Prevent further
	// processing of this request
	//
	pDE->TimeRemaining = -1;
	pDE->DeviceStatus = XxReadStatus( pDE );
	return FALSE;
}
