//
// Headers
//
#include <windows.h>
#include <stdio.h>
#include <conio.h>

//
// Definitions
//
#define MESSAGE_COUNT	10
#define MESSAGE_TEXT	"The owls are not what they seem."

//
// Global data
//
HANDLE hDevice;

//
// Forward declarations of local functions...
//
DWORD 
Writer(
	LPVOID lpvParam
	);

static VOID
ErrorMessage( 
	LPTSTR lpOrigin, 
	DWORD dwMessageId
	);

//
// Main entry point...
//
VOID __cdecl
main(
	int argc,
	char *argv[]
	)
{
	HANDLE hThread;
	DWORD dwErrorCode;
	DWORD dwThreadId;
	DWORD dwBytesRead;
	TCHAR szBuffer[ 100 ];
	OVERLAPPED overlap;
	BOOL status;
	DWORD i;

	if( argc < 2 )
	{
		printf( "Usage: XXTEST \\\\.\\dev\n" );
		ExitProcess( ERROR_SUCCESS );
	}

 	hDevice = CreateFile(
				argv[1],
				GENERIC_READ | GENERIC_WRITE,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_FLAG_OVERLAPPED,
				NULL );
	if( hDevice == INVALID_HANDLE_VALUE )
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "CreateFile", dwErrorCode );
		ExitProcess( dwErrorCode );
	}

	//
	// Create a thread to do the writing
	//
	hThread = CreateThread(
				NULL, 			// no security
				0,				// default stack size
				(LPTHREAD_START_ROUTINE) Writer,
				NULL,			// no parameter
				0,				// flags -- create running
				&dwThreadId );
	if( hThread == INVALID_HANDLE_VALUE )
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "CreateThread", dwErrorCode );
		ExitProcess( dwErrorCode );
	}

	//
	// Set up asynch operation (otherwise, we won't
	// be able to use the same handle in two threads).
	//
	overlap.Offset = 0;
	overlap.OffsetHigh = 0;
	overlap.hEvent = CreateEvent(
						NULL,
						TRUE,
						FALSE,
						NULL );

	//
	// Read the 10 messages sent from the other side
	//
	for( i=0; i<MESSAGE_COUNT; i++ )
	{
		status = ReadFile( 
					hDevice, 
					szBuffer, 
					sizeof( szBuffer ),
					&dwBytesRead,
					&overlap );
		if( !status &&
			GetLastError() != ERROR_IO_PENDING )
		{
			dwErrorCode = GetLastError();
			ErrorMessage( "ReadFile", dwErrorCode );
			break;
		}

		//
		// Wait for the read to complete
		//
		status = GetOverlappedResult(
					hDevice,
					&overlap,
					&dwBytesRead,
					TRUE );	
		if( !status )
		{
			dwErrorCode = GetLastError();
			ErrorMessage( "ReadFile", dwErrorCode );
			break;
		}

		szBuffer[ dwBytesRead ] = '\0';
		printf( "Msg #%2d [%d bytes]: %s\n",
					i+1,
					dwBytesRead,
					szBuffer );
	}

	//
	// All done with this; get rid of it
	//
	CloseHandle( overlap.hEvent );

	//
	// Hang around until the writer is finished
	//
	WaitForSingleObject( hThread, INFINITE );
	CloseHandle( hThread );
	CloseHandle( hDevice );
	ExitProcess( ERROR_SUCCESS );
}

//
// This function writes messages to the device
//
DWORD 
Writer(
	LPVOID lpvParam
	)
{
	LPCTSTR szBuffer = MESSAGE_TEXT;
	DWORD dwBytesWritten;
	DWORD dwErrorCode;
	OVERLAPPED overlap;
	BOOL status;
	DWORD i;

	//
	// Set up accoutrements of overlapped I/O
	//
	overlap.Offset = 0;
	overlap.OffsetHigh = 0;
	overlap.hEvent = CreateEvent(
						NULL,
						TRUE,
						FALSE,
						NULL );

	//
	// Wait 10 seconds before starting
	//
	Sleep( 10000 );

	//
	// Write several messages
	//
	for( i=0; i<MESSAGE_COUNT; i++ )
	{
		status = WriteFile( 
					hDevice, 
					szBuffer, 
					strlen( szBuffer ),
					&dwBytesWritten,
					&overlap );
		if( !status &&
			GetLastError() != ERROR_IO_PENDING )
		{
			dwErrorCode = GetLastError();
			ErrorMessage( "WriteFile", dwErrorCode );
			ExitThread( dwErrorCode );
		}

		//
		// Wait for the write to complete
		//
		status = GetOverlappedResult(
					hDevice,
					&overlap,
					&dwBytesWritten,
					TRUE );	
		if( !status )
		{
			dwErrorCode = GetLastError();
			ErrorMessage( "WriteFile", dwErrorCode );
			CloseHandle( overlap.hEvent );
			ExitThread( dwErrorCode );
		}

		//
		// Introduce a little timing delay. This is
		// necessary because the driver is using the
		// FIFO timeout as a way to detect end of
		// transmission.
		//
		Sleep( 100 );
	}

	//
	// Get rid of Event object
	//
	CloseHandle( overlap.hEvent );

	return( ERROR_SUCCESS );
}


//
// This helper routine converts a system-service error
// code into a text message and prints it on StdOutput
//
static VOID
ErrorMessage( 
	LPTSTR lpOrigin,
	DWORD dwMessageId
	)
{
	LPTSTR msgBuffer;		// string returned from system
	DWORD cBytes;			// length of returned string

	cBytes = FormatMessage(
				FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_ALLOCATE_BUFFER,
				NULL,
				dwMessageId,
				MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),
				(TCHAR *)&msgBuffer,
				500,
				NULL );
	if( msgBuffer ) {
		msgBuffer[ cBytes ] = TEXT('\0');
		printf( "Error: %s -- %s\n", lpOrigin, msgBuffer );
		LocalFree( msgBuffer );
	}
	else {
		printf( "FormatMessage error: %d\n", GetLastError());
	}
}
