//++
// File Name:
//		dispatch.c
//
// Contents:
//		Dispatch routines for driver
//--

//
// Driver-specific header files
//
#include "xxdriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, save space
// by making various routines pageable
//    
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, XxDispatchOpen )
#pragma alloc_text( page, XxDispatchClose )
#pragma alloc_text( page, XxDispatchDeviceControl )
#endif //ALLOC_PRAGMA

//++
// Function:
//		XxDispatchOpen
//
// Description:
//		This function dispatches CreateFile
//		requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchOpen(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	//
	// Before the packet goes back to the
	// I/O Manager, log a message.
	//
	XxReportEvent(
		LOG_LEVEL_DEBUG,
		XX_MSG_OPENING_HANDLE,
		XX_ERRORLOG_DISPATCH,
		(PVOID)pDO,				// Device object
		Irp,
		NULL, 0,				// No dump data
		NULL, 0 );				// No strings

	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return STATUS_SUCCESS;
}

//++
// Function:
//		XxDispatchClose
//
// Description:
//		This function dispatches CloseHandle
//		requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchClose(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	//
	// Before the packet goes back to the
	// I/O Manager, log a message.
	//
	XxReportEvent(
		LOG_LEVEL_DEBUG,
		XX_MSG_CLOSING_HANDLE,
		XX_ERRORLOG_DISPATCH,
		(PVOID)pDO,				// Device object
		Irp,
		NULL, 0,				// No dump data
		NULL, 0 );				// No strings

	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return STATUS_SUCCESS;
}

//++
// Function:
//		XxDispatchDeviceControl
//
// Description:
//		This function dispatches DeviceIoControl
//		requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchDeviceControl(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{
	NTSTATUS status;
	ULONG DumpData[1];

	PWSTR Strings[2] = {
			L"human",
			L"fly"
	};

	PIO_STACK_LOCATION IrpStack =
		IoGetCurrentIrpStackLocation( Irp );

	ULONG ControlCode = 
		IrpStack->Parameters.
			DeviceIoControl.IoControlCode;

	switch( ControlCode )
	{
		case IOCTL_XX_START_TRANSPORTER:
			//
			// Log a message with dump data
			//
			DumpData[0] = 2;

			XxReportEvent(
				LOG_LEVEL_NORMAL,
				XX_MSG_MULTIPLE_OCCUPANCY,
				XX_ERRORLOG_DISPATCH,
				(PVOID)pDO,
				Irp,
				DumpData,	// Array of data
				1,			// Number of ULONGs
				NULL, 0 );	// No strings

			//
			// Log a message with strings
			//
			XxReportEvent(
				LOG_LEVEL_NORMAL,
				XX_MSG_MERGING_DNA,
				XX_ERRORLOG_DISPATCH,
				(PVOID)pDO,
				Irp,
				NULL, 0,	// No dump data
				Strings,	// Insertion strings
				2 );		// Number of strings

			status = STATUS_SUCCESS;
			Irp->IoStatus.Information = 0;
			break;

		//
		// Not one we recognize. Error.
		//
		default:
			status = STATUS_INVALID_PARAMETER; 
 			Irp->IoStatus.Information = 0;
			break;
	}

	//
	// Get rid of this request
	//
	Irp->IoStatus.Status = status;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return status;
}
