// File Name:
//		unload.c
//
// Contents:
//		Routines that perform driver cleanup
//

#include "xxdriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, make the cleanup
// pageable so it won't be hanging around all the
// time
//
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, XxDriverUnload )
#endif // ALLOC_PRAGMA

//++
// Function:
//		XxDriverUnload
//
// Description:
//		This function releases any resources 
//		allocated during driver initialization.
//
// Arguments:
//		A pointer to the Driver object
//
// Return Value:
//		(None)
//--
VOID
XxDriverUnload(
	IN PDRIVER_OBJECT DriverObject
	)
{
	PDEVICE_OBJECT pDevObj;
	
	UNICODE_STRING linkName;

	//
	// Get pointer to Device object
	//	
	pDevObj = DriverObject->DeviceObject;

	//
   	// Form the Win32 symbolic link name.
   	//
	RtlInitUnicodeString( 
		&linkName, 
		XX_WIN32_DEVICE_NAME );
	    
	//        
	// Remove symbolic link from Object
	// namespace...
	//
	IoDeleteSymbolicLink( &linkName );

	//
	// Get rid of the Device object...
	//
	IoDeleteDevice( pDevObj );

	//
	// Put a shutdown message in the Event Log
	//
	XxReportEvent(
		LOG_LEVEL_DEBUG,
		XX_MSG_DRIVER_STOPPING,
		XX_ERRORLOG_UNLOAD,
		(PVOID)DriverObject,
		NULL,					// No IRP
		NULL, 0,				// No dump data
		NULL, 0 );				// No strings
}

