//++
// File Name:
//		hardware.h
//
// Contents:
//		Hardware-specific details
//--
#ifndef __XX_HARDWARE__
#define __XX_HARDWARE__

//
// Define which types of hardware resources this
// device needs to allocate. Here, the device needs
// ports and interrupt resources, but not memory or
// DMA. If one device uses more than one of the same
// resource, (two disjoint ranges of I/O space, for 
// example) set the symbol equal to the number of
// items it uses.
//
#define	XX_PORT_RESOURCE_ITEMS			1
#define	XX_INTERRUPT_RESOURCE_ITEMS		1
#define	XX_MEMORY_RESOURCE_ITEMS		0
#define	XX_DMA_RESOURCE_ITEMS			0

//
// Define the number of CM_PARTIAL_RESOURCE_DESCRIPTOR
// items needed for each piece of hardware belonging
// to this driver.
//
#define XX_RESOURCE_ITEMS_PER_DEVICE		\
(	(XX_PORT_RESOURCE_ITEMS)			+	\
	(XX_INTERRUPT_RESOURCE_ITEMS)		+	\
	(XX_MEMORY_RESOURCE_ITEMS)			+	\
	(XX_DMA_RESOURCE_ITEMS)					)

//
// Default values for hardware
//
#define XX_DEF_PORT			0x0100
#define XX_DEF_PORT_SPAN	2
#define XX_DEF_IRQ			7
#define XX_DEF_DMA			5
#define	XX_MAX_DMA_LENGTH	0x20000

//
// Define device registers as relative offsets
//
#define XX_STATUS_REG	0	// Status register
#define XX_CTRL_REG		1	// Control register

//
// Status register bits:
//
#define XX_STS_IRQ 	0x04	// 1 Interrupt request
#define XX_STS_ERR	0X08	// 1 Error

//
// Control register bits:
//
#define XX_CTL_DMA_GO	0x01 // 1 Start DMA operation
#define XX_CTL_INTENB	0x02 // 1 Enable interrupts

//
// Define access macros for registers. Each macro takes
// a pointer to a Device Extension as an argument
//
#define XxReadStatus( pDevExt )				\
(READ_PORT_UCHAR(							\
  (pDevExt)->PortBase + XX_STATUS_REG ))

#define XxReadControl( pDevExt )			\
(READ_PORT_UCHAR(							\
  (pDevExt)->PortBase + XX_CTRL_REG ))

#define XxWriteControl( pDevExt, bData )	\
(WRITE_PORT_UCHAR(							\
  (pDevExt)->PortBase + XX_CTRL_REG, (bData)))

//
// A macro for testing the goodness of
// a hardware status byte
//
#define XX_STS_OK( DevStatus )				\
(((DevStatus) & XX_STS_ERR) == 0 )

#endif // __XX_HARDWARE__

