// File Name:
//		unload.c
//
// Contents:
//		Routines that perform driver cleanup
//

#include "yydriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, make the cleanup
// pageable so it won't be hanging around all the
// time
//
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, YyDriverUnload )
#endif // ALLOC_PRAGMA

//++
// Function:
//		YyDriverUnload
//
// Description:
//		This function releases any resources 
//		allocated during driver initialization.
//
// Arguments:
//		A pointer to the Driver object
//
// Return Value:
//		(None)
//--
VOID
YyDriverUnload(
	IN PDRIVER_OBJECT DriverObject
	)
{
	PDEVICE_OBJECT ThisFilterDevice;
	PDEVICE_OBJECT NextFilterDevice;

	PDEVICE_EXTENSION FilterExtension;
	
	//
	// Get pointer to first Device object
	//	
	ThisFilterDevice = DriverObject->DeviceObject;

	//
	// Get rid of them all
	//
	while( ThisFilterDevice != NULL )
	{
		FilterExtension = 
			ThisFilterDevice->DeviceExtension;

		//
		// Disconnect from the target Device object
		//
		IoDetachDevice( 
			FilterExtension->TargetDevice );

		//
		// Delete filter Device object and move
		// on to the next one in the list.
		//
		NextFilterDevice = 
			ThisFilterDevice->NextDevice;
    
		IoDeleteDevice( ThisFilterDevice );

		ThisFilterDevice = NextFilterDevice;
	}
}

