//++
// File Name:
//		transfer.c
//
// Contents:
//		Data transfer routines for dummy driver
//--

//
// Driver-specific header files...
//
#include "xxdriver.h"

//
// Forward declarations of local functions
//
// (None)

//++
// Function:
//		XxStartIo
//
// Description:
//		This function is responsible for doing any
//		function-code specific pre-processing and 
//		then starting the actual data transfer.
//
// Arguments:
//		Pointer to the Device object
//		Pointer to the IRP for this request
//
// Return Value:
//		(None)
//--
VOID
XxStartIo(
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp
	)
{
	ULONG ControlCode;

	PIO_STACK_LOCATION	IrpStack = 
		IoGetCurrentIrpStackLocation( Irp );

	PDEVICE_EXTENSION pDE =
		DeviceObject->DeviceExtension;

	switch( IrpStack->MajorFunction )
	{
		case IRP_MJ_READ:

			Irp->IoStatus.Information =
				IrpStack->Parameters.Read.Length;

			Irp->IoStatus.Status = 
						STATUS_SUCCESS;
			break;
			
		case IRP_MJ_WRITE:

			Irp->IoStatus.Information =
				IrpStack->Parameters.Write.Length;

			Irp->IoStatus.Status = 
						STATUS_SUCCESS;
			break;

		case IRP_MJ_DEVICE_CONTROL:
			ControlCode = 
				IrpStack->Parameters.
							DeviceIoControl.
								IoControlCode;

			switch( ControlCode )
			{
				case IOCTL_XX_FUNCTION1:
				case IOCTL_XX_FUNCTION2:

					Irp->IoStatus.Status = 
								STATUS_SUCCESS;

					Irp->IoStatus.Information = 0;
					break;

				//
				// Unknown IOCTL...
				//
				default:
					Irp->IoStatus.Status = 
						STATUS_INVALID_DEVICE_REQUEST;
					Irp->IoStatus.Information = 0;
					break;
			}
			break;

		//
		// Should never get here -- just get rid
		// of the packet...
		//	
		default:
			Irp->IoStatus.Status =
					STATUS_INVALID_DEVICE_REQUEST;
			Irp->IoStatus.Information = 0;
			break;
	}

	//
	// Start the delay timer
	//
	KeSetTimer(
		&pDE->IrpDelayTimer,
		pDE->IrpDelayInterval,
		&pDE->IrpDelayDpc );
}

//++
// Function:
//		XxIrpDelayTimerDpc
//
// Description:
//		This function completes the current IRP
//		after waiting a respectable interval.
//
// Arguments:
//		Pointer to the Dpc object (Not used)
//		Pointer to the Device object
//		SystemArgument1 (Not Used)
//		SystemArgument2 (Not Used)
//
// Return Value:
//		(None)
//--
VOID
XxIrpDelayTimerDpc(
	IN PKDPC Dpc,
	IN PVOID Context,
	IN PVOID SystemArgument1,
	IN PVOID SystemArgument2
	)
{
	PDEVICE_OBJECT DeviceObject = Context;

	IoCompleteRequest( 
		DeviceObject->CurrentIrp, 
		IO_NO_INCREMENT );

	IoStartNextPacket( 
		DeviceObject, 
		FALSE );
}


	