// File Name:
//		unload.c
//
// Contents:
//		Routines that perform driver cleanup
//

#include "xxdriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, make the cleanup
// pageable so it won't be hanging around all the
// time
//
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, XxDriverUnload )
#endif // ALLOC_PRAGMA

//++
// Function:
//		XxDriverUnload
//
// Description:
//		This function releases any resources 
//		allocated during driver initialization.
//
// Arguments:
//		A pointer to the Driver object
//
// Return Value:
//		(None)
//--
VOID
XxDriverUnload(
	IN PDRIVER_OBJECT DriverObject
	)
{
	PDEVICE_OBJECT ThisDevice;
	PDEVICE_OBJECT NextDevice;
	UNICODE_STRING linkName;

	//        
	// Remove symbolic links from Object
	// namespace...
	//
	RtlInitUnicodeString( 
		&linkName, 
		XX_WIN32_DEVICE_NAME1 );
	
	IoDeleteSymbolicLink( &linkName );

	RtlInitUnicodeString( 
		&linkName, 
		XX_WIN32_DEVICE_NAME2 );

	IoDeleteSymbolicLink( &linkName );
	    
	//
	// Get pointer to first Device object
	//	
	ThisDevice = DriverObject->DeviceObject;

	while( ThisDevice != NULL )
	{
		NextDevice = ThisDevice->NextDevice;
		IoDeleteDevice( ThisDevice );
		ThisDevice = NextDevice;
	}
}

