// File Name:
//		xxdriver.h
//
// Contents:
//		Constants, structures, and function
//		declarations specific to this driver.
//
#ifndef __XX_DRIVER__
#define __XX_DRIVER__

//
// Header files
//
#include <ntddk.h>
#include <devioctl.h>

#include "..\inc\xxioctl.h"

//
// Various constants
//
#define	XX_NT_DEVICE_NAME1		L"\\Device\\Xx0"
#define	XX_WIN32_DEVICE_NAME1	L"\\DosDevices\\XX1"

#define	XX_NT_DEVICE_NAME2		L"\\Device\\Xx1"
#define	XX_WIN32_DEVICE_NAME2	L"\\DosDevices\\XX2"

#define	XX_DRIVER_NAME			L"XXDRIVER"
#define XX_MAX_BUFFER_SIZE		300
#define XX_IRP_DELAY_INTERVAL	500 // Milliseconds

//++
// Description:
//		Driver-defined structure used to hold 
//		miscellaneous device information.
//
// Access:
//		Allocated from NON-PAGED POOL
//		Available at any IRQL
//--
typedef struct _DEVICE_EXTENSION {
	PDEVICE_OBJECT DeviceObject; // Back pointer
	ULONG BytesRequested;

	KDPC IrpDelayDpc;	  // Components of the
	KTIMER IrpDelayTimer; //  IRP delay mechanism
	LARGE_INTEGER IrpDelayInterval;
	
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;


//
// Prototypes for globally defined functions...
//
// init.c
//
NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath
	);

//
// unload.c
//
VOID
XxDriverUnload(
	IN PDRIVER_OBJECT DriverObject
	);

//
// dispatch.c
//
NTSTATUS
XxDispatchOpen(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

NTSTATUS
XxDispatchClose(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

NTSTATUS
XxDispatchRead(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

NTSTATUS
XxDispatchWrite(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

NTSTATUS
XxDispatchDeviceIoControl(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

//
// transfer.c
//
VOID
XxStartIo(
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp
	);

VOID
XxIrpDelayTimerDpc(
	IN PKDPC Dpc,
	IN PVOID Context,
	IN PVOID SystemArgument1,
	IN PVOID SystemArgument2
	);

#endif	// __XX_DRIVER__

