//
// Headers
//
#include <windows.h>
#include <stdio.h>
#include <winioctl.h>

#include "..\inc\xxioctl.h"

//
// Miscellaneous definitions...
//
#define BUFFER_SIZE 300

//
// Forward declarations of local functions...
//
static VOID
ErrorMessage( 
	LPTSTR lpOrigin, 
	DWORD dwMessageId
	);

//
// Main entry point...
//
VOID __cdecl
main(
	int argc,
	char *argv[]
	)
{
	HANDLE hDevice;
	DWORD dwErrorCode;
	DWORD dwTransferSize;
	TCHAR szBuffer[ BUFFER_SIZE ];
	XX_BUFFER_SIZE_INFO BufferInfo;

	if( argc < 2 )
	{
		printf( "Usage: XXTEST \\\\.\\dev\n" );
		ExitProcess( ERROR_SUCCESS );
	}

	hDevice = CreateFile(
				argv[1],
				GENERIC_READ |
					GENERIC_WRITE,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL,
				NULL );
	if( hDevice == INVALID_HANDLE_VALUE )
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "CreateFile", dwErrorCode );
		ExitProcess( dwErrorCode );
	}
	printf( "Opened %s\n", argv[1] );

	//
	// DeviceIoControl test...
	//
	// Buffer-size information...
	//
	if( !DeviceIoControl(
			hDevice,
			IOCTL_XX_GET_MAX_BUFFER_SIZE,
			NULL,
			0,
			&BufferInfo,
			sizeof( BufferInfo ),
			&dwTransferSize,
			NULL ))
	{
		dwErrorCode = GetLastError();
		ErrorMessage( 
			"DeviceIoControl(IOCTL_XX_GET_MAX_BUFFER_SIZE):",
			dwErrorCode );
		ExitProcess( dwErrorCode );
	}

	if( BufferInfo.MaxReadLength == XX_NO_BUFFER_LIMIT )
		printf( "Unlimited input-buffer size.\n" );
	else
		printf( 
			"Input-buffer size limited to %d bytes.\n",
			BufferInfo.MaxReadLength );

	if( BufferInfo.MaxWriteLength == XX_NO_BUFFER_LIMIT )
		printf( "Unlimited output-buffer size.\n" );
	else
		printf( 
			"Output-buffer size limited to %d bytes.\n",
			BufferInfo.MaxWriteLength );

	//
	// Generic IOCTLs...
	//
	if( !DeviceIoControl(
			hDevice,
			IOCTL_XX_FUNCTION1,
			NULL, 0,
			NULL, 0,
			&dwTransferSize,
			NULL ))
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "DeviceIoControl(IOCTL_XX_FUNCTION1):",
			dwErrorCode );
		ExitProcess( dwErrorCode );
	}
	printf( "DeviceIoControl: IOCTL_XX_FUNCTION1.\n" );

	if( !DeviceIoControl(
			hDevice,
			IOCTL_XX_FUNCTION2,
			NULL, 0,
			NULL, 0,
			&dwTransferSize,
			NULL ))
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "DeviceIoControl(IOCTL_XX_FUNCTION2):",
			dwErrorCode );
		ExitProcess( dwErrorCode );
	}
	printf( "DeviceIoControl: IOCTL_XX_FUNCTION2.\n" );

	//
	// ReadFile test...
	//
	if( !ReadFile(
			hDevice,
			szBuffer,
			sizeof( szBuffer ),
			&dwTransferSize,
			NULL ))
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "ReadFile", dwErrorCode );
		ExitProcess( dwErrorCode );
	}
	printf( "ReadFile: %d bytes transferred.\n", dwTransferSize );

	//
	// WriteFile test...
	//
	if( !WriteFile(
			hDevice,
			szBuffer,
			sizeof( szBuffer ),
			&dwTransferSize,
			NULL ))
	{
		dwErrorCode = GetLastError();
		ErrorMessage( "WriteFile", dwErrorCode );
		ExitProcess( dwErrorCode );
	}
	printf( "WriteFile: %d bytes transferred.\n", dwTransferSize );

	//
	// End of tests...
	//
	printf( "All tests completed successfully.\n" );

	CloseHandle( hDevice );
	ExitProcess( ERROR_SUCCESS );
}


//
// This helper routine converts a system-service error
// code into a text message and prints it on StdOutput
//
static VOID
ErrorMessage( 
	LPTSTR lpOrigin,	// Indicates error location
	DWORD dwMessageId	// ERROR_XXX code value
	)
{
	LPTSTR msgBuffer;		// string returned from system
	DWORD cBytes;			// length of returned string

	cBytes = FormatMessage(
				FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_ALLOCATE_BUFFER,
				NULL,
				dwMessageId,
				MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),
				(TCHAR *)&msgBuffer,
				500,
				NULL );
	if( msgBuffer ) {
		msgBuffer[ cBytes ] = TEXT('\0');
		printf( "Error: %s -- %s\n", lpOrigin, msgBuffer );
		LocalFree( msgBuffer );
	}
	else {
		printf( "FormatMessage error: %d\n", GetLastError());
	}
}
