//++
// File Name:
//		init.c
//
// Contents:
//		Driver initialization code
//--

//
// Driver-specific header files...
//
#include "xxdriver.h"

//
// Forward declarations of local functions
//
// (None)

//
// If the compiler can handle it, make the DriverEntry
// routine discardable, so that it doesn't waste space
//    
#ifdef ALLOC_PRAGMA
#pragma alloc_text( init, DriverEntry )
#endif // ALLOC_PRAGMA

//++
// Function:
//		DriverEntry
//
// Description:
//		This function initializes the driver, and
//		creates various NT objects needed to process
//		I/O requests.
//
// Arguments:
//		Pointer to the Driver object
//		Registry path string for driver service key
//
// Return Value:
//		NTSTATUS signaling success or failure
//--
NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath
	)
{
	NTSTATUS status;

	PDEVICE_OBJECT pDevObj;
    PDEVICE_EXTENSION pDevExt;

	UNICODE_STRING deviceName;
	UNICODE_STRING linkName;

	//
	// Export other driver entry points...
	//
 	DriverObject->DriverUnload = XxDriverUnload;
	
	DriverObject->MajorFunction[ IRP_MJ_CREATE ] =
		XxDispatchOpenClose;
	DriverObject->MajorFunction[ IRP_MJ_CLOSE ] =
		XxDispatchOpenClose;

	//
   	// Form the NT device name...
   	//
	RtlInitUnicodeString(
		&deviceName,
		XX_NT_DEVICE_NAME );

    //
    // Create a Device object...
    //
	status = IoCreateDevice(
				DriverObject,
				sizeof( DEVICE_EXTENSION ),
				&deviceName,
				FILE_DEVICE_UNKNOWN,
				0,
				TRUE,
				&pDevObj );

	if( !NT_SUCCESS( status )) {
		return status;
    }

 	//
	// Choose a buffering mechanism
	//
	pDevObj->Flags |= DO_BUFFERED_IO;

	//
    // Initialize the device extension structure
    //
	pDevExt = pDevObj->DeviceExtension;
    pDevExt->DeviceObject = pDevObj;

	//
   	// Form the Win32 symbolic link name.
   	//
	RtlInitUnicodeString( 
		&linkName, 
		XX_WIN32_DEVICE_NAME );
		
	//        
	// Create a symbolic link so our device is
	// visible to Win32...
	//
 	status = IoCreateSymbolicLink(
 				&linkName, &deviceName );
    
	//
    // See if the symbolic link was created...
    //
	if( !NT_SUCCESS( status )) {
    	IoDeleteDevice( pDevObj );
    }

	return status;
}

