//++
// File Name:
//		dispatch.c
//
// Contents:
//		Dispatch routines for driver
//--

//
// Driver-specific header files
//
#include "xxdriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, save space
// by making various routines pageable
//    
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, XxDispatchOpenClose )
#endif //ALLOC_PRAGMA

//++
// Function:
//		XxDispatchOpenClose
//
// Description:
//		This function dispatches CreateFile and
//		CloseHandle requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchOpenClose(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{

	KdPrint(( "Entering XxDispatchOpenClose...\n" ));

	//
	// See if it's time to croak
	//
	XxTryToCrash( XX_CRASH_DISPATCH_OPEN );

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return STATUS_SUCCESS;
}

//++
// Function:
//		XxDispatchWrite
//
// Description:
//		This function dispatches WriteFile
//		requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchWrite(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{

	KdPrint(( "Entering XxDispatchWrite...\n" ));

	//
	// See if it's time to croak
	//
	XxTryToCrash( XX_CRASH_DISPATCH_WRITE );

	//
	// Send packet to Start I/O
	//
	IoMarkIrpPending( Irp );
	IoStartPacket( pDO, Irp, 0, NULL );

	return STATUS_PENDING;
}



