// File Name:
//		xxdriver.h
//
// Contents:
//		Constants, structures, and function
//		declarations specific to this driver.
//
#ifndef __XX_DRIVER__
#define __XX_DRIVER__

//
// Header files
//
#include <ntddk.h>

//
// Various constants
//
#define	XX_NT_DEVICE_NAME		L"\\Device\\Crash0"
#define	XX_WIN32_DEVICE_NAME	L"\\DosDevices\\CRASH1"
#define	XX_DRIVER_NAME			L"XXDRIVER"

#define XX_TIMEOUT_VALUE		15

//
// Places where the driver can crash...
//
#define XX_CRASH_NONE			0
#define XX_CRASH_DRIVER_ENTRY	1
#define XX_CRASH_DISPATCH_OPEN	2
#define XX_CRASH_DISPATCH_WRITE	3
#define XX_CRASH_START_IO		4
#define XX_CRASH_ISR			5
#define XX_CRASH_DPC			6
#define XX_CRASH_TIMER			7
#define XX_CRASH_DRIVER_UNLOAD	8

#define XX_DEFAULT_CRASH_TYPE XX_CRASH_NONE

//++
// Description:
//		Driver-defined structure used to hold 
//		crash information.
//
// Access:
//		Allocated from NON-PAGED POOL
//		Available at any IRQL
//--
typedef struct _CRASH_BLOCK {
	ULONG Stuff;
	ULONG MoreStuff;
	ULONG EvenMoreStuff;
} CRASH_BLOCK, *PCRASH_BLOCK;

//++
// Description:
//		Driver-defined structure used to hold 
//		miscellaneous device information.
//
// Access:
//		Allocated from NON-PAGED POOL
//		Available at any IRQL
//--
typedef struct _DEVICE_EXTENSION {
	PDEVICE_OBJECT DeviceObject; // Back pointer

	//
	// Miscellaneous Kernel objects
	//
	KDPC CompletionDPC;

	//
	// Working variables...
	//
	ULONG BytesRequested;
	ULONG BytesRemaining;
	PUCHAR pBuffer;

	ULONG TimeoutCounter;

	//
	// Things for catching bugchecks
	//
	KBUGCHECK_CALLBACK_RECORD CallbackRecord;
	CRASH_BLOCK CrashData;
	
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

//
// Prototypes for globally defined functions...
//
//
// init.c
//
NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath
	);

//
// unload.c
//
VOID
XxDriverUnload(
	IN PDRIVER_OBJECT DriverObject
	);

//
// dispatch.c
//
NTSTATUS
XxDispatchOpenClose(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

NTSTATUS
XxDispatchWrite(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	);

//
// transfer.c
//
VOID
XxStartIo(
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp
	);

VOID
XxCompleteRequest(
	IN PKDPC Dpc,
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp,
	IN PVOID Context
	);

VOID
XxTimer(
	IN PDEVICE_OBJECT DeviceObject,
	IN PVOID Context
	);

//
// crash.c
//
VOID
XxGetCrashPoint(
	VOID
	);

VOID
XxTryToCrash(
	IN ULONG CurrentLocation
	);

VOID
XxBugCheckCallback(
	IN PVOID Buffer,
	IN ULONG Length
	);

#endif	// __XX_DRIVER__

