//++
// File Name:
//		dispatch.c
//
// Contents:
//		Dispatch routines for driver
//--

//
// Driver-specific header files
//
#include "xxdriver.h"

//
// Forward declarations of local routines
//
// (None)

//
// If the compiler can handle it, save space
// by making various routines pageable
//    
#ifdef ALLOC_PRAGMA
#pragma alloc_text( page, XxDispatchOpenClose )
#pragma alloc_text( page, XxDispatchDeviceControl )
#endif // ALLOC_PRAGMA

//++
// Function:
//		XxDispatchOpenClose
//
// Description:
//		This function dispatches CreateFile and
//		CloseHandle requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchOpenClose(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return STATUS_SUCCESS;
}

//++
// Function:
//		XxDispatchDeviceControl
//
// Description:
//		This function dispatches DeviceIoControl
//		requests from Win32
//
// Arguments:
//		Pointer to Device object
//		Pointer to IRP for this request
//
// Return Value:
//		This function returns STATUS_XXX
//--
NTSTATUS
XxDispatchDeviceControl(
	IN PDEVICE_OBJECT pDO,
	IN PIRP Irp
	)
{
	NTSTATUS status;

	PIO_STACK_LOCATION IrpStack =
		IoGetCurrentIrpStackLocation( Irp );

	PDEVICE_EXTENSION pDE =
		(PDEVICE_EXTENSION)pDO->DeviceExtension;

	ULONG ControlCode = 
		IrpStack->Parameters.
			DeviceIoControl.IoControlCode;

	ULONG BufferSize = 
		IrpStack->Parameters.
			DeviceIoControl.OutputBufferLength;

	PXX_PERF_DATA pBuffer =	
			(PXX_PERF_DATA)Irp->
				AssociatedIrp.SystemBuffer;

	switch( ControlCode )
	{
		case IOCTL_XX_GET_PERF_DATA:
			if(	BufferSize < sizeof( XX_PERF_DATA ))
			{
				status = STATUS_BUFFER_TOO_SMALL;
 				Irp->IoStatus.Information = 0;
			}
			else
			{
				pBuffer->ByteLength = 
							sizeof( XX_PERF_DATA );

				pBuffer->Interrupts = 
							pDE->InterruptCounter;
				pBuffer->Operations = 
							pDE->OperationCounter;

				//
				// Increment the counters just to see
				// some change in the display. In a
				// situation, other parts of the 
				// driver would be maintaining these
				// counters.
				//
				if(( pDE->OperationCounter & 3 ) == 0)
					pDE->InterruptCounter += 117;
				else
					pDE->InterruptCounter += 21;

				if( pDE->InterruptCounter & 1 )
					pDE->OperationCounter +=
					pDE->InterruptCounter;
				else
					pDE->OperationCounter += 71;

				status = STATUS_SUCCESS;
 				Irp->IoStatus.Information = 
							sizeof( XX_PERF_DATA );
			}
			break;
		//
		// Not one we recognize. Error.
		//
		default:
			status = STATUS_INVALID_PARAMETER; 
 			Irp->IoStatus.Information = 0;
			break;
	}

	//
	// Get rid of this request
	//
	Irp->IoStatus.Status = status;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return status;
}



