/******************************************************************************
*                                                                                                                          *
* Copyright 1993 Adaptec, Inc.,  All Rights Reserved.                         *
*                                                                                                                          *
* This software contains the valuable trade secrets of Adaptec.  The             *
* software is protected under copyright laws as an unpublished work of        *
* Adaptec.  Notice is for informational purposes only and does not imply      *
* publication.  The user of this software may make copies of the software     *
* for use with parts manufactured by Adaptec or under license from Adaptec    *
* and for no other use.                                                                                *
*                                                                                                                          *
******************************************************************************/

/******************************************************************************
*
*  Driver Name:  ASPIDRV.SYS
*
*  Version:       1.0
*
*  Source Code:  ASPIDRV.C ASPIDRV.H MAKEFILE
*
*  Base Code #:  537805-00
*
*  Description:  This driver, ASPIDRV.SYS is a sample device driver that
*    works in conjunction with an OS/2 application to provide SRBs to an
*    OS/2 ASPI manager (i.e OS2ASPI.DMD).
*
*                 - You should have an understanding of writing OS/2 device
*                   drivers to fully understand this driver.
*
*                 - There are NO command line switches.  The driver will
*                   automatically attach itself to OS2ASPI.DMD (if present)
*         and display a status message on the screen.
*
*                 - Only single-thread requests are supported.
*
*                 - Scatter/gather lists are NOT supported. A virtual pointer to the
*         data buffer is passed in the IOCtl data buffer pointer and the 
*         driver places a physical pointer in the SRB.
*
*                 - Initialization code and data are thrown away after INIT
*                   to minimize resident memory requirements.
*
*  History:
*
*     07/15/93 - JHL - v1.0 submitted.
*
*     03/02/95 - JHL - v1.1 submitted.
*
******************************************************************************/
#define INCL_NOBASEAPI
#define INCL_NOPMAPI
#include <os2.h>

#include <strat2.h>

#include <devcmd.h>
#include <devclass.h>
#include <reqpkt.h>

#include <iorb.h>
#include <addcalls.h>
#include <scsi.h>
#include <dhcalls.h>

#include <aspi.h>
#include "aspidrv.h"

extern APIRET APIENTRY DosPutMessage(USHORT hf, 
				     USHORT cbMsg, 
				     PCHAR pchMsg);
extern void STRATEGY();                                  //name of strategy routine in DDSTART */

extern char *ASPIDriverInstalled;
extern char *ASPIDriverAborted;

static strlen(PSZ s);

void FAR _loadds ASPI_Post(USHORT DataSeg,
			   PASPI_SRB_HEADER pSRBH);
BOOL CreatePhysicalDataPointer(PASPI_SRB_EXECUTE_IO pSRB,
			       PUCHAR dataBufferPtr);
int Init(PRPINITIN pRPI);
int ASPIDriverEntry(PRPH rp);

PFN   Device_Help = 0;                                   // Pointer to Dev_Help
ATTACHAREA ASPIData;                                     // OS2ASPI entry information
ULONG dataHandle;                                        // Handle for locked data buffer
ULONG parameterHandle;                                   // Handle for locked memory

/******************************************************
*                                                     *
*  Function Name : ASPIDriverEntry                    *
*                                                     *
*  Function Description : This function receives all  *
*  request packets from the application and routes    *
*  them based on the command and function values.     *
*                                                     *
*  An ASPI request must be a generic IOCTL with       *
*  function type ASPI_REQ. Once these requests are    *
*  identified the SRB is updated to include any new   *
*  information that is required and then the request  *
*  is forwarded to OS2ASPI.DMD                        *
*                                                     *
*  Function Input :                                   *
*    rp : A pointer to the driver's request packet    *
*                                                     *
*  Function Output : This function returns STDON if   *
*  the command is successfully handled and an error   *
*  code if a problem is detected.                     *
*                                                     *
******************************************************/
int ASPIDriverEntry(PRPH rp)

   {
   ULONG parameterPtr;                                   // Temporary variable for physical ptr
   PRP_GENIOCTL rpIOCTL;
   USHORT result = STDON;

   switch (rp->Cmd)
      {
      case RPINIT:                                       //Initialize the driver
	 return (Init((PRPINITIN) rp));

      case RPOPEN:                                       //Open the driver
	 return(STDON);

      case RPCLOSE:                                      //Close the driver
	 return(STDON);

      case RPIOCTL:                 
	 rpIOCTL = (PRP_GENIOCTL) rp;
	 if (rpIOCTL->Category == ASPI_REQ)              // Is it an ASPI call?
	    {
	    if (DevHelp_Lock(SELECTOROF(rpIOCTL->ParmPacket),
			     1,                          // long term lock
			     0,                          // block until available
			     (PULONG) &parameterHandle))
	       return(STDON | STERR | ERROR_GEN_FAILURE);

	    switch (rpIOCTL->Function)                   // Which ASPI call?
	       {
	       case ASPI_CMD_ADAPTER_INQUIRY:
	       case ASPI_CMD_GET_DEVICE_TYPE:
		  // Verify access to the SRB
		  if (DevHelp_VerifyAccess(SELECTOROF(rpIOCTL->ParmPacket),
					   sizeof(ASPI_SRB_HEADER),
					   OFFSETOF(rpIOCTL->ParmPacket),
					   VERIFY_READWRITE))
		     {
		     DevHelp_UnLock(parameterHandle);
		     result = STDON | STERR | ERROR_GEN_FAILURE;
		     } 
		  else
		     // Send the ASPI request to OS2ASPI.DMD
		     {
		     (*ASPIData.protectedEntryPoint)(ASPIData.protectedDS,
						     (PASPI_SRB_HEADER) rpIOCTL->ParmPacket);
		     // Unlock the parameter packet
		     DevHelp_UnLock(parameterHandle);
		     }
		  break;                  
	    
	       case ASPI_CMD_EXECUTE_IO:
		  // Verify access to the SRB
		  if (DevHelp_VerifyAccess(SELECTOROF(rpIOCTL->ParmPacket),
					   sizeof(ASPI_SRB_EXECUTE_IO),
					   OFFSETOF(rpIOCTL->ParmPacket),
					   VERIFY_READWRITE))
		     {
		     DevHelp_UnLock(parameterHandle);
		     result = STDON | STERR | ERROR_GEN_FAILURE;
		     } 
		  else               
		     // Create a physical pointer to the SRB
		     if (DevHelp_VirtToPhys(rpIOCTL->ParmPacket,
					    &parameterPtr))
			{
			DevHelp_UnLock(parameterHandle);
			result = (STDON | STERR | ERROR_GEN_FAILURE);
			}
		     else
			{
			// Copy the physical SRB pointer into the SRB
			((PASPI_SRB_EXECUTE_IO)(rpIOCTL->ParmPacket))->ppSRB = parameterPtr;
		     
			// Copy the post routine address into the SRB
			((PASPI_SRB_EXECUTE_IO)(rpIOCTL->ParmPacket))->PM_PostAddress = ASPI_Post;

			// Copy the post routine data selector to the SRB
			((PASPI_SRB_EXECUTE_IO)(rpIOCTL->ParmPacket))->PM_DataSeg = ASPIData.protectedDS;

			// Create a physical pointer to the data buffer
			if ((CreatePhysicalDataPointer((PASPI_SRB_EXECUTE_IO) rpIOCTL->ParmPacket,
						       rpIOCTL->DataPacket)) == FALSE)
			   {
			   DevHelp_UnLock(parameterHandle);
			   result = (STDON | STERR | ERROR_GEN_FAILURE);
			   }
			else
			   {
			   // Send the ASPI request to OS2ASPI.DMD
			   (*ASPIData.protectedEntryPoint)(ASPIData.protectedDS,
							   (PASPI_SRB_HEADER) rpIOCTL->ParmPacket);

			   DISABLE
			   // Make sure the command has completed
			   if (((PASPI_SRB_EXECUTE_IO)(rpIOCTL->ParmPacket))->SRBHdr.ASPIReqFlags & ASPI_REQFLAG_POST_ENABLE)
			      {
			      while ((volatile) ((PASPI_SRB_EXECUTE_IO)(rpIOCTL->ParmPacket))->SRBHdr.ASPIStatus == ASPI_STATUS_IN_PROGRESS)
				 {
				 DevHelp_ProcBlock(parameterPtr,
						   (ULONG) -1,
						   WAIT_IS_INTERRUPTABLE);
				 DISABLE
				 }
			      }

			   ENABLE

			   // Unlock the data buffer
			   DevHelp_UnLock(dataHandle);

			   // Unlock the parameter packet. 
			   DevHelp_UnLock(parameterHandle);
			   }
			}
		  break;

	       case ASPI_CMD_ABORT_IO:
		  // Verify access to the SRB
		  if (DevHelp_VerifyAccess(SELECTOROF(rpIOCTL->ParmPacket),
					   sizeof(ASPI_SRB_ABORT_IO),
					   OFFSETOF(rpIOCTL->ParmPacket),
					   VERIFY_READWRITE))
		     {
		     DevHelp_UnLock(parameterHandle);
		     result = STDON | STERR | ERROR_GEN_FAILURE;
		     } 
		  else
		     {               
		     // Create a physical pointer to the SRB
		     if (DevHelp_VirtToPhys(rpIOCTL->ParmPacket,
					    &parameterPtr))
			{
			DevHelp_UnLock(parameterHandle);
			result = STDON | STERR | ERROR_GEN_FAILURE;
			}
		     else
			{
			// Copy the physical SRB pointer into the SRB
			((PASPI_SRB_ABORT_IO)(rpIOCTL->ParmPacket))->ppSRB = parameterPtr;
		     
			// Send the ASPI request to OS2ASPI.DMD
			(*ASPIData.protectedEntryPoint)(ASPIData.protectedDS,
							(PASPI_SRB_HEADER) rpIOCTL->ParmPacket);

			// Unlock the parameter packet. 
			DevHelp_UnLock(parameterHandle);
			}
		     }
		  break;
	       case ASPI_CMD_RESET_DEVICE:
		  // Verify access to the SRB
		  if (DevHelp_VerifyAccess(SELECTOROF(rpIOCTL->ParmPacket),
					   sizeof(ASPI_SRB_RESET_DEVICE),
					   OFFSETOF(rpIOCTL->ParmPacket),
					   VERIFY_READWRITE))
		     {
		     DevHelp_UnLock(parameterHandle);
		     result = STDON | STERR | ERROR_GEN_FAILURE;
		     } 
		  else
		     {
		     // Copy the post routine address into the SRB
		     ((PASPI_SRB_RESET_DEVICE)(rpIOCTL->ParmPacket))->PM_PostAddress = ASPI_Post;

		     // Copy the post routine data selector to the SRB
		     ((PASPI_SRB_RESET_DEVICE)(rpIOCTL->ParmPacket))->PM_DataSeg = ASPIData.protectedDS;

		     // Send the ASPI request to OS2ASPI.DMD
		     (*ASPIData.protectedEntryPoint)(ASPIData.protectedDS,
						     (PASPI_SRB_HEADER) rpIOCTL->ParmPacket);
			
		     DISABLE
		     // Make sure the command has completed
		     if (((PASPI_SRB_RESET_DEVICE)(rpIOCTL->ParmPacket))->SRBHdr.ASPIReqFlags & ASPI_REQFLAG_POST_ENABLE)
			{
			while ((volatile) ((PASPI_SRB_RESET_DEVICE)(rpIOCTL->ParmPacket))->SRBHdr.ASPIStatus == ASPI_STATUS_IN_PROGRESS)
			   {
			   DevHelp_ProcBlock(parameterPtr,
					     (ULONG) -1,
					     WAIT_IS_INTERRUPTABLE);
			   DISABLE
			   }
			}
		     ENABLE

		     // Unlock the parameter packet. 
		     DevHelp_UnLock(parameterHandle);
		     }
		  break;

	       case ASPI_CMD_SET_ADAPTER_PARMS:
	       default:                                  // No other functions are supported
		  {
		  DevHelp_UnLock(parameterHandle);
		  result = STDON | STERR | ERROR_BAD_COMMAND;
		  }
	       }
	    return(result);
	    }
	 else                                            // No other categories are supported
	    return (STDON | STERR | ERROR_BAD_COMMAND);

      default:                                           // An unknown command was issued to the driver
	 return(STDON | STERR | ERROR_BAD_COMMAND);
      }
   }
   
/******************************************************
*                                                     *
*  Function Name : CreatePhysicalDataPointer          *
*                                                     *
*  Function Description : This function converts the  *
*  virtual data pointer into a physical pointer and   *
*  copies it into the appropriate SRB field.          *
*                                                     *
*  Function Input :                                   *
*     pSRB - A pointer to the SRB being processed.    *
*                                                     *
*  Function Output : This function returns TRUE if a  *
*  valid pointer is created and FALSE if there is     *
*  an error detected.                                 *
*                                                     *
******************************************************/
BOOL CreatePhysicalDataPointer(PASPI_SRB_EXECUTE_IO pSRB, 
			       PUCHAR dataBufferPtr)

   {

   // If the command does not transfer data, no pointer is required
   if (pSRB->DataXferLen == 0)
      {
      pSRB->ppDataBuffer = 0;
      return(TRUE);
      }

   if (!(DevHelp_Lock(SELECTOROF(dataBufferPtr),
		      1,                          // long term lock
		      0,                          // block until available
		      (PULONG) &dataHandle)))
      if (!(DevHelp_VerifyAccess(SELECTOROF(dataBufferPtr),
				 (USHORT) pSRB->DataXferLen,
				 OFFSETOF(dataBufferPtr),
				 VERIFY_READWRITE)))
	  if (!(DevHelp_VirtToPhys(dataBufferPtr,
				   &(pSRB->ppDataBuffer))))

	       return(TRUE);
	  else
	       {
	       DevHelp_UnLock(dataHandle);
	       return(FALSE);
	       }
      else
	 {
	 DevHelp_UnLock(dataHandle);
	 return(FALSE);
	 }
   else
      {
      return(FALSE);
      }
   }


/******************************************************
*                                                     *
*  Function Name : ASPI_Post                          *
*                                                     *
*  Function Description : This function is called     *
*  whenever an ASPI request is completed by the ASPI  *
*  device manager.                                    *
*                                                     *
*  Currently all that it does is wake up the thread   *
*  which sent the SRB. It is the thread's             *
*  responsibility to check the status of the SRB and  *
*  perform any actions based on that status.          *
*                                                     *
*  Function Input :                                   *
*     DataSeg - The data selector of this driver      *
*     pSRBH - A pointer to the completed SRB          *
*                                                     *
*  Function Output :                                  *
*                                                     *
******************************************************/
void FAR _loadds ASPI_Post(USHORT DataSeg,PASPI_SRB_HEADER pSRBH)
   {
   USHORT awakeCount;

   // The following code assumes that requests are single threaded. 
   // If you are going to allow multiple requests to be active
   // simultaneously then you need to create data structures that will
   // record the handle and virtual data pointer for each request. 

   // Wake up the thread which made the request.
   DevHelp_ProcRun((ULONG) pSRBH, 
		   &awakeCount);

   // Return control to OS2ASPI.DMD
   }


/******************************************************
*                                                     *
*  Function Name : Init                               *
*                                                     *
*  Function Description : This function formally      *
*  sets up the device driver so that it can be        *
*  loaded into memory and accessed by subsequent      *
*  calls from application programs.                   *
*                                                     *
*  Function Input :                                   *
*    rp : A pointer to the driver's request packet    *
*                                                     *
*  Function Output : This function returns RPDONE if  *
*  it successfully completes all its tasks and a      *
*  result code indicating the problem that occurred   *
*  if it cannot be loaded.                            *
*                                                     *
******************************************************/
int Init(PRPINITIN pRPI)
   {
   char *SCSIManager = "SCSIMGR$ ";
   PRPINITOUT   pRPO = (PRPINITOUT) pRPI;

   // Initialize DevHelp entry point
   Device_Help = pRPI->DevHlpEP;

   // Attach to OS2ASPI.DMD
   if (DevHelp_AttachDD(SCSIManager,
			(NPBYTE) &ASPIData))
   
      {
      // The driver failed to install
      DosPutMessage(1,                                // Handle is STDOUT
		    strlen(ASPIDriverAborted),        // Message length
		    ASPIDriverAborted);               // Message ptr
      pRPO->CodeEnd = 0;
      pRPO->DataEnd = 0;

      return(RPDONE | RPERR | ERROR_BAD_COMMAND);     
      }
   else
      {
      // The driver was succesfully installed
      DosPutMessage(1,                                // Handle is STDOUT
		    strlen(ASPIDriverInstalled),      // Message length
		    ASPIDriverInstalled);             // Message ptr

      pRPO->CodeEnd = (USHORT) &Init;
      pRPO->DataEnd = (USHORT) &parameterHandle + sizeof(PULONG);

      return(RPDONE);                                 
      }
   }

// Functions that replace C library functions unavailable to a device driver

/******************************************************
*                                                     *
*  Function Name : strlen                             *
*                                                     *
*  Function Description : This function calculates the*
*  length of a string.                                *
*                                                     *
*  Function Input :                                   *
*    s : A pointer to the string                      *
*                                                     *
*  Function Output : This function returns the length *
*  of the string.                                     *
*                                                     *
******************************************************/
static strlen(PSZ s)
   {
   int   i = 0;

   while (*s++) 
      i++;

   return (i);
   }
