#define WANTVXDWRAPS

#include <basedef.h>
#include <vmm.h>
#include <vmmreg.h>
#include <vxdwraps.h>
#include <vwin32.h>
#include <winerror.h>

/* from ASMSTUFF.ASM */
extern ULONG grabmem_PageReserve(ULONG page, ULONG npages, ULONG flags);
extern ULONG grabmem_PageCommitPhys(ULONG page, ULONG npages, ULONG physpage, ULONG flags);
extern ULONG grabmem_PageCommitContig(ULONG page, ULONG npages, ULONG flags, ULONG alignmask, ULONG minphys, ULONG maxphys);

#include "grabmemd.h"

#define MAJOR_VERSION	1
#define MINOR_VERSION	0




unsigned CommitBiggestPhysChunk( ULONG linpage, unsigned max_pages )
	{
	// ---------------------------------------------------------------
	// Commits the largest possible run of physically contiguous pages
	// to the specified linear address
	// ---------------------------------------------------------------

	while (max_pages > 0)
		{
		if ( grabmem_PageCommitContig( linpage, max_pages, 
									   PC_USER | PC_WRITEABLE, 
									   0, 0, 0xFFFFFFFF ) != -1L )
			{
			return max_pages;
			}

		if (max_pages >= 8)
			max_pages = max_pages - (max_pages / 8);
		else if (max_pages >= 4)
			max_pages = max_pages - (max_pages / 4);
		else
			max_pages = max_pages - 1;
		}

	return 0;
	}


void *AllocateBuffer( unsigned maxpages, 
					  unsigned max_chunks, 
					  int allow_partial, 
					  DWORD *num_pages_allocated, 
					  DWORD *num_chunks_allocated )
	{
	// ---------------------------------------------------------------
	// Allocates a buffer with at most 'max_chunks' non-contiguous
	// runs of memory.
	// ---------------------------------------------------------------

	long linpage;
	unsigned npages=0;

	*num_pages_allocated = 0;
	*num_chunks_allocated = 0;

	linpage = grabmem_PageReserve( PR_PRIVATE, maxpages, PR_FIXED | PR_4MEG );
	if (linpage == -1L)
		{
		return 0;
		}

	// Now map physical pages into the linear address range
	npages = 0;
	while ( (max_chunks > 0) && (npages < maxpages) )
		{
		unsigned n = CommitBiggestPhysChunk( linpage/4096 + npages,
											 maxpages - npages);
		if (n == 0)
			break;

		npages += n;
		max_chunks--;
		*num_chunks_allocated += 1;
		}

	if ( (npages == 0) || ((npages < maxpages) && !allow_partial) )
		{
		// May be causing a GPF!
		_PageFree((void *) linpage,0);
		return 0;
		}

	*num_pages_allocated = npages;
	return (void *) linpage;
	}



DWORD _stdcall GRABMEM_W32_DeviceIOControl(DWORD	dwService,
					 DWORD	dwDDB,
					 DWORD	hDevice,
					 DIOCPARAMETERS *lpDIOCParms)
	{
	GRABMEM_CMD *p;
    DWORD dwRetVal = 0;

    switch (dwService) 
		{
		case GRABMEM_EXEC_FUNCTION:
			p = (GRABMEM_CMD *) lpDIOCParms->lpvInBuffer;
			switch (p->Command)
				{
				case GRABMEM_CMD_GetVersion:
					((GRABMEM_VERSION *)p)->Major = MAJOR_VERSION;
					((GRABMEM_VERSION *)p)->Minor = MINOR_VERSION;
					break;

				case GRABMEM_CMD_AllocateBuffer:
					{
					unsigned maxpages = (p->RequestedLength + 4095) / 4096;

					p->VirtualAddress = AllocateBuffer(	maxpages,
														p->MaxChunks, 
														p->Flags & GRABMEM_FLAG_AllowPartialAllocation,
														&p->AllocatedLength,
														&p->NumChunks);
					p->AllocatedLength *= 4096;
					}
					break;

				case GRABMEM_CMD_FreeBuffer:
					// May be causing a GPF
					_PageFree(p->VirtualAddress,0);
					break;
				}

			dwRetVal = 0;
			break;

		default:
			dwRetVal = 0;
			break;
		}

    return dwRetVal;
	}

DWORD _stdcall GRABMEM_Dynamic_Init(VOID)
    {
    return (1);
    }

DWORD _stdcall GRABMEM_Dynamic_Exit(VOID)
    {
    return (1);
    }

