
#include <stdio.h>
#include <windows.h>

#include "grabmemd.h"

GRABMEM_VERSION	GrabmemVersionInfo;
GRABMEM_CMD		GrabmemAllocInfo;

int CheckMem(void *bufp, unsigned nbytes)
	{
	unsigned long *p = (unsigned long *) bufp;
	unsigned n = nbytes / sizeof(unsigned long);
	unsigned i;
	for (i=0; i<n; i++)
		*(p+i) = i;

	for (i=0; i<n; i++)
		{
		if (*(p+i) != i)
			{
			printf("Memory problem at offset %u!\n",i*4);
			return 0;
			}
		}

	printf("Memory test passed\n");
	return 1;
	}

int main(int argc, char *argv[])
	{
    HANDLE      hVXD = 0;
    DWORD       cbBytesReturned;
    DWORD       dwErrorCode;
	long		nbytes = 4096;


	if (argc >= 2)
		{
		nbytes = strtol(argv[1],NULL,0);
		if (nbytes <= 0)
			{
			printf("Minimum allocation size of 1 byte\n");
			return 3;
			}
		}


    // Dynamically load and prepare to call GRABMEM
    // The CREATE_NEW flag is not necessary
    hVXD = CreateFile("\\\\.\\GRABMEM.VXD",0,0,0,CREATE_NEW,FILE_FLAG_DELETE_ON_CLOSE, 0);

    if ( hVXD == INVALID_HANDLE_VALUE )
		{
        dwErrorCode = GetLastError();
        if ( dwErrorCode == ERROR_NOT_SUPPORTED )
			{
            printf("Unable to open VxD, \n device does not support DeviceIOCTL\n");
			return 5;
			}
        else
			{
            printf("Unable to open VxD, Error code: %lx\n", dwErrorCode);
			return 4;
			}
		}
	

	// Get GRABMEM version number
	memset(&GrabmemVersionInfo,0,sizeof(GrabmemVersionInfo));
	GrabmemVersionInfo.Command = GRABMEM_CMD_GetVersion;
	DeviceIoControl( hVXD,
					 GRABMEM_EXEC_FUNCTION,
					 &GrabmemVersionInfo,sizeof(GrabmemVersionInfo),
					 &GrabmemVersionInfo,sizeof(GrabmemVersionInfo),
					 &cbBytesReturned, NULL);
		{
		printf("GRABMEM Version %u.%02u\n",GrabmemVersionInfo.Major,GrabmemVersionInfo.Minor);
		}


	// Allocate 'nbytes' of contiguous, locked memory
	printf("Allocating %lu bytes of contiguous, locked memory\n",nbytes);
	memset(&GrabmemAllocInfo,0,sizeof(GrabmemAllocInfo));
	GrabmemAllocInfo.Command = GRABMEM_CMD_AllocateBuffer;
	GrabmemAllocInfo.Flags = 0;
	GrabmemAllocInfo.RequestedLength = nbytes;
	GrabmemAllocInfo.MaxChunks = 16;
	DeviceIoControl( hVXD, 
					 GRABMEM_EXEC_FUNCTION,
					 &GrabmemAllocInfo,sizeof(GrabmemAllocInfo),
					 &GrabmemAllocInfo,sizeof(GrabmemAllocInfo),
					 &cbBytesReturned, NULL);

	if (GrabmemAllocInfo.VirtualAddress == NULL)
		{
		printf("Allocation failed!\n");
		return 6;
		}
	else
		{
		printf("Allocation successful!\n");
		printf("Virtual Address:  %08X\n",GrabmemAllocInfo.VirtualAddress);
		printf("Allocated Length: %08X\n",GrabmemAllocInfo.AllocatedLength);
		printf("Number of Chunks: %u\n",GrabmemAllocInfo.NumChunks);
		fflush(stdout);

		// memset(GrabmemAllocInfo.VirtualAddress,0,GrabmemAllocInfo.AllocatedLength);
		CheckMem(GrabmemAllocInfo.VirtualAddress,GrabmemAllocInfo.AllocatedLength);

		// Free the contiguous, locked memory
		GrabmemAllocInfo.Command = GRABMEM_CMD_FreeBuffer;
		DeviceIoControl( hVXD, 
						 GRABMEM_EXEC_FUNCTION,
						 &GrabmemAllocInfo,sizeof(GrabmemAllocInfo),
						 &GrabmemAllocInfo,sizeof(GrabmemAllocInfo),
						 &cbBytesReturned, NULL);
		}


	// Dynamically UNLOAD the GRABMEM VxD
	CloseHandle(hVXD);

	return(0);
}
